// BlackJack1.java

import ch.aplu.jcardgame.*;
import ch.aplu.jgamegrid.*;
import ch.aplu.util.Monitor;

public class BlackJack1 extends CardGame
{
  public enum Suit
  {
    SPADES, HEARTS, DIAMONDS, CLUBS
  }

  public enum Rank
  {
    ACE, KING, QUEEN, JACK, TEN, NINE, EIGHT, SEVEN, SIX, FIVE, FOUR, THREE, TWO
  }
  // ------------------------ Locations -----------------
  private Location upperLocation = new Location(300, 120);
  private Location bottomLocation = new Location(300, 470);
  private Location middleLocation = new Location(300, 300);
  private Location hideLocation = new Location(-500, -500);
  private Location buttonLocation = new Location(300, 575);
  private Location gameOverLocation = new Location(300, 300);
  // ----------------------------------------------------

  private Deck deck = new Deck(Suit.values(), Rank.values(), "cover");
  private Hand dealer = new Hand(deck);
  private Hand player = new Hand(deck);
  private Actor gameOverActor = new Actor("sprites/gameover.gif");
  private GGButton standButton = new GGButton("sprites/standBtn.gif");
  private GGButton restartButton = new GGButton("sprites/restartBtn.gif");
  private Hand talon;

  public BlackJack1()
  {
    super(600, 600, 30);
    setTitle("BlackJack - Extended Version");
    setStatusText("Dealing out. Please wait...");
    initButtons();
    while (true)
    {
      init();
      Monitor.putSleep();
    }
  }

  private void showHands()
  {
    RowLayout rowLayout = new RowLayout(bottomLocation, 350);
    player.setView(this, rowLayout);
    player.draw();
    RowLayout rowLayout1 = new RowLayout(upperLocation, 350);
    dealer.setView(this, rowLayout1);
    dealer.draw();
    talon.setVerso(true);
    talon.setView(this, new StackLayout(middleLocation));
    talon.draw();
  }

  private void init()
  {
    talon = deck.dealingOut(0, 0, false)[0];
    showHands();

    Card card = talon.getLast();
    talon.transfer(card, player, true);
    card.setVerso(false);
    card = talon.getLast();
    card.setVerso(true);
    talon.transfer(card, dealer, true);
    card = talon.getLast();
    talon.transfer(card, player, true);
    card.setVerso(false);
    card = talon.getLast();
    talon.transfer(card, dealer, true);
    card.setVerso(false);

    CardAdapter cardAdapter = new CardAdapter()
    {
      public void leftDoubleClicked(Card card)
      {
        card.transfer(player, true);
        card.setVerso(false);
        int value = getHandValue(player);
        setStatusText(value);
        if (value > 21)
        {
          gameOver("You bust! Your card values " + value
            + " exceed 21  ---  Please click to restart.");
        }
      }
    };

    talon.addCardListener(cardAdapter);
    setStatusText(getHandValue(player));
    standButton.setLocation(buttonLocation);
    talon.setTouchEnabled(true);
    setStatusText(getHandValue(player));
  }

  private void initButtons()
  {
    restartButton.addButtonListener(new GGButtonListener()
    {
      public void buttonClicked(GGButton button)
      {
        restartButton.setLocation(hideLocation);
        cleanUp();
        Monitor.wakeUp();
      }

      public void buttonReleased(GGButton button)
      {
      }

      public void buttonPressed(GGButton button)
      {
      }
    });
    addActor(restartButton, hideLocation);
    restartButton.setRefreshEnabled(false);

    standButton.addButtonListener(new GGButtonListener()
    {
      public void buttonClicked(GGButton button)
      {
        standButton.setLocation(hideLocation);
        dealersTurn();
        evaluateRound();
      }

      public void buttonReleased(GGButton button)
      {
      }

      public void buttonPressed(GGButton button)
      {
      }
    });
    addActor(standButton, hideLocation);
    standButton.setRefreshEnabled(false);
  }

  private void cleanUp()
  {
    gameOverActor.removeSelf();
    talon.removeAll(false);
    player.removeAll(false);
    dealer.removeAll(false);
  }

  private void evaluateRound()
  {
    int dealerValue = getHandValue(dealer);
    int playerValue = getHandValue(player);
    String score = playerValue + " (you) versus "
      + dealerValue + " (dealer)  ---  Please click to restart.";

    if (dealerValue > 21)
      gameOver("Dealer bust! His card values " + dealerValue
        + " exceed 21  ---  Please click to restart.");
    else
    {
      if (playerValue > dealerValue)
        gameOver("You won! " + score);
      if (playerValue == dealerValue)
        gameOver("Tie! " + score);
      else
        gameOver("You lost! " + score);
    }
  }

  private void gameOver(String reason)
  {
    dealer.setVerso(false);
    setStatusText(reason);
    talon.setTouchEnabled(false);
    addActor(gameOverActor, gameOverLocation);
    standButton.setLocation(hideLocation);
    restartButton.setLocation(buttonLocation);
  }

  private void dealersTurn()
  {
    talon.setTargetArea(new TargetArea(dealer.getHandLocation()));
    while (getHandValue(dealer) < 17)
    {
      Card newCard = talon.getLast();
      newCard.transfer(dealer, true);
      newCard.setVerso(false);
    }
  }

  private int getHandValue(Hand hand)
  {
    int value = 0;
    for (Card card : hand.getCardList())
    {
      switch ((Rank)card.getRank())
      {
        case TWO:
          value += 2;
          break;
        case THREE:
          value += 3;
          break;
        case FOUR:
          value += 4;
          break;
        case FIVE:
          value += 5;
          break;
        case SIX:
          value += 6;
          break;
        case SEVEN:
          value += 7;
          break;
        case EIGHT:
          value += 8;
          break;
        case NINE:
          value += 9;
          break;
        case TEN:
          value += 10;
          break;
        case JACK:
          value += 10;
          break;
        case QUEEN:
          value += 10;
          break;
        case KING:
          value += 10;
          break;
        case ACE:
          if (value + 11 > 21)
            value += 1;
          else
            value += 11;
          break;
      }
    }
    return value;
  }

  private void setStatusText(int value)
  {
    setStatusText(value + " points in your hand.    "
      + "Double-click card talon to Hit or press Stand.");
  }

  public static void main(String[] args)
  {
    new BlackJack1();
  }
}
